"""
Clase híbrida para números binarios - ADAPTADA AL NUEVO SISTEMA
Ahora usa IntBase como clase base universal
"""
from sympy_Base import SympyClassBase
import re

# Importación dinámica de IntBase desde el registro
def get_intbase_class():
    """Obtiene la clase IntBase del registro de tipos"""
    try:
        from type_registry import get_registered_base_context
        context = get_registered_base_context()
        return context.get('IntBase')
    except ImportError:
        return None


class Class_Bin(SympyClassBase):
    """Clase híbrida para números binarios basada en IntBase"""
    
    def __init__(self, value_input):
        """Inicialización de Bin usando IntBase como base"""
        
        # Obtener IntBase dinámicamente
        IntBase = get_intbase_class()
        if IntBase is None:
            raise ImportError("IntBase no disponible en el registro de tipos")
        
        if isinstance(value_input, IntBase):
            # value es IntBase (ya tokenizado desde 2#valor)
            if value_input.base == 2:
                self.int_base = value_input
            else:
                # Convertir de otra base a binario
                if not value_input.has_symbols:
                    bin_value = IntBase._convert_to_base_string(value_input._numeric_value, 2)
                    self.int_base = IntBase(bin_value, 2)
                else:
                    # Mantener simbólico para análisis algebraico
                    self.int_base = value_input
        elif isinstance(value_input, str):
            # Conversión desde string
            clean_value = value_input
            if clean_value.startswith('0b'):
                clean_value = clean_value[2:]
            self.int_base = IntBase(clean_value, 2)
        elif isinstance(value_input, (int, float)) or hasattr(value_input, '__int__'):
            # Conversión desde entero (incluyendo sympy.Integer)
            try:
                int_val = int(value_input)
                bin_value = IntBase._convert_to_base_string(int_val, 2)
                self.int_base = IntBase(bin_value, 2)
            except (ValueError, TypeError):
                raise TypeError(f"No se puede convertir {type(value_input)} a entero")
        else:
            raise TypeError(f"Bin no puede inicializarse con {type(value_input)}")
        
        # Llamar al constructor base con el valor del IntBase
        super().__init__(self.int_base.value, str(self.int_base))
    
    def __str__(self):
        """Representación string para display"""
        if self.int_base.has_symbols:
            return f"0b{self.int_base.value_str}"
        else:
            return f"0b{self.int_base._numeric_value:08b}"
    
    def _sympystr(self, printer):
        """Representación SymPy string"""
        return str(self)
    
    def __repr__(self):
        return f"Bin({self.int_base.value_str})"
    
    # ========== MÉTODOS DE CONVERSIÓN ==========
    
    def toDecimal(self):
        """Convierte a decimal"""
        if self.int_base.has_symbols:
            return self.int_base._symbolic_value  # Expresión simbólica
        else:
            return self.int_base._numeric_value
    
    def toHex(self):
        """Convierte a hexadecimal"""
        if self.int_base.has_symbols:
            return f"Hex({self.int_base._symbolic_value})"
        else:
            hex_value = IntBase._convert_to_base_string(self.int_base._numeric_value, 16)
            return f"0x{hex_value}"
    
    def toOctal(self):
        """Convierte a octal"""
        if self.int_base.has_symbols:
            return f"Oct({self.int_base._symbolic_value})"
        else:
            oct_value = IntBase._convert_to_base_string(self.int_base._numeric_value, 8)
            return f"0o{oct_value}"
    
    def toBase(self, base):
        """Convierte a cualquier base"""
        return self.int_base.to_base(base)
    
    # ========== OPERADORES ARITMÉTICOS (delegados a IntBase) ==========
    
    def __add__(self, other):
        """Suma delegada a IntBase"""
        if isinstance(other, Class_Bin):
            result_intbase = self.int_base + other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base + other
        elif isinstance(other, int):
            result_intbase = self.int_base + other
        else:
            return NotImplemented
        
        return Class_Bin(result_intbase)
    
    def __sub__(self, other):
        """Resta delegada a IntBase"""
        if isinstance(other, Class_Bin):
            result_intbase = self.int_base - other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base - other
        elif isinstance(other, int):
            result_intbase = self.int_base - other
        else:
            return NotImplemented
        
        return Class_Bin(result_intbase)
    
    def __mul__(self, other):
        """Multiplicación delegada a IntBase"""
        if isinstance(other, Class_Bin):
            result_intbase = self.int_base * other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base * other
        elif isinstance(other, int):
            result_intbase = self.int_base * other
        else:
            return NotImplemented
        
        return Class_Bin(result_intbase)
    
    def __truediv__(self, other):
        """División delegada a IntBase"""
        if isinstance(other, Class_Bin):
            result_intbase = self.int_base / other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base / other
        elif isinstance(other, int):
            result_intbase = self.int_base / other
        else:
            return NotImplemented
        
        return Class_Bin(result_intbase)
    
    def __mod__(self, other):
        """Módulo delegado a IntBase"""
        if isinstance(other, Class_Bin):
            result_intbase = self.int_base % other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base % other
        elif isinstance(other, int):
            result_intbase = self.int_base % other
        else:
            return NotImplemented
        
        return Class_Bin(result_intbase)
    
    # Operadores reversos
    def __radd__(self, other):
        return self.__add__(other)
    
    def __rmul__(self, other):
        return self.__mul__(other)
    
    # ========== MÉTODOS ESTÁTICOS ==========
    
    @staticmethod
    def Helper(input_str):
        """Ayuda contextual para Bin"""
        if re.match(r"^\s*Bin\b", input_str, re.IGNORECASE):
            return ('Ej: 2#1010 (se convierte automáticamente), Bin(10)\n'
                   'Funciones: toDecimal(), toHex(), toOctal(), toBase(n)')
        return None

    @staticmethod
    def PopupFunctionList():
        """Lista de métodos sugeridos para autocompletado de Bin"""
        return [
            ("toDecimal", "Convierte a decimal"),
            ("toHex", "Convierte a hexadecimal"),
            ("toOctal", "Convierte a octal"),
            ("toBase", "Convierte a base específica"),
        ]

    @staticmethod
    def get_tokenization_patterns():
        """
        Define patrones adicionales de tokenización específicos para Bin
        
        Returns:
            List[Dict]: Lista de reglas de tokenización (opcional)
        """
        return [
            {
                'pattern': r'\b0b([01]+)\b',
                'replacement': lambda match: f'Bin("{match.group(1)}")',
                'priority': 6,  # Prioridad media, después de IntBase pero específica
                'description': 'Binario con prefijo 0b: 0b1010, 0b11'
            }
        ]


# ========== FUNCIÓN DE REGISTRO ==========

def register_classes_in_module():
    """
    Devuelve una lista de clases definidas en este módulo para ser registradas.
    NOTA: Ya no se registra para corchetes, el tokenizador maneja 2#valor automáticamente
    """
    return [
        ("Bin", Class_Bin, "SympyClassBase", {
            "add_lowercase": True,
            "supports_brackets": False,  # Ya no usa corchetes
            "description": "Números binarios (usar 2#valor para tokenización automática)"
        }),
    ]