"""
Clase híbrida para números hexadecimales - ADAPTADA AL NUEVO SISTEMA
Ahora usa IntBase como clase base universal
"""
from sympy_Base import SympyClassBase
import re

# Importación dinámica de IntBase desde el registro
def get_intbase_class():
    """Obtiene la clase IntBase del registro de tipos"""
    try:
        from type_registry import get_registered_base_context
        context = get_registered_base_context()
        return context.get('IntBase')
    except ImportError:
        return None


class Class_Hex(SympyClassBase):
    """Clase híbrida para números hexadecimales basada en IntBase"""
    
    def __init__(self, value_input):
        """Inicialización de Hex usando IntBase como base"""
        
        # Obtener IntBase dinámicamente
        IntBase = get_intbase_class()
        if IntBase is None:
            raise ImportError("IntBase no disponible en el registro de tipos")
        
        if isinstance(value_input, IntBase):
            # value es IntBase (ya tokenizado desde 16#valor)
            if value_input.base == 16:
                self.int_base = value_input
            else:
                # Convertir de otra base a hexadecimal
                if not value_input.has_symbols:
                    hex_value = IntBase._convert_to_base_string(value_input._numeric_value, 16)
                    self.int_base = IntBase(hex_value, 16)
                else:
                    # Mantener simbólico para análisis algebraico
                    self.int_base = value_input
        elif isinstance(value_input, str):
            # Conversión desde string
            clean_value = value_input
            if clean_value.startswith('0x'):
                clean_value = clean_value[2:]
            self.int_base = IntBase(clean_value, 16)
        elif isinstance(value_input, (int, float)) or hasattr(value_input, '__int__'):
            # Conversión desde entero (incluyendo sympy.Integer)
            try:
                int_val = int(value_input)
                hex_value = IntBase._convert_to_base_string(int_val, 16)
                self.int_base = IntBase(hex_value, 16)
            except (ValueError, TypeError):
                raise TypeError(f"No se puede convertir {type(value_input)} a entero")
        else:
            raise TypeError(f"Hex no puede inicializarse con {type(value_input)}")
        
        # Llamar al constructor base con el valor del IntBase
        super().__init__(self.int_base.value, str(self.int_base))
    
    def __str__(self):
        """Representación string para display"""
        if self.int_base.has_symbols:
            return f"0x{self.int_base.value_str}"
        else:
            return f"0x{self.int_base._numeric_value:02X}"
    
    def _sympystr(self, printer):
        """Representación SymPy string"""
        return str(self)
    
    def __repr__(self):
        return f"Hex({self.int_base.value_str})"
    
    # ========== MÉTODOS DE CONVERSIÓN ==========
    
    def toDecimal(self):
        """Convierte a decimal"""
        if self.int_base.has_symbols:
            return self.int_base._symbolic_value  # Expresión simbólica
        else:
            return self.int_base._numeric_value
    
    def toBinary(self):
        """Convierte a binario"""
        if self.int_base.has_symbols:
            # Para valores simbólicos, crear expresión de conversión
            return f"Bin({self.int_base._symbolic_value})"
        else:
            bin_value = IntBase._convert_to_base_string(self.int_base._numeric_value, 2)
            return f"0b{bin_value}"
    
    def toOctal(self):
        """Convierte a octal"""
        if self.int_base.has_symbols:
            return f"Oct({self.int_base._symbolic_value})"
        else:
            oct_value = IntBase._convert_to_base_string(self.int_base._numeric_value, 8)
            return f"0o{oct_value}"
    
    def toBase(self, base):
        """Convierte a cualquier base"""
        return self.int_base.to_base(base)
    
    # ========== OPERADORES ARITMÉTICOS (delegados a IntBase) ==========
    
    def __add__(self, other):
        """Suma delegada a IntBase"""
        IntBase = get_intbase_class()
        
        if isinstance(other, Class_Hex):
            result_intbase = self.int_base + other.int_base
        elif IntBase and isinstance(other, IntBase):
            result_intbase = self.int_base + other
        elif isinstance(other, int):
            result_intbase = self.int_base + other
        else:
            return NotImplemented
        
        return Class_Hex(result_intbase)
    
    def __sub__(self, other):
        """Resta delegada a IntBase"""
        if isinstance(other, Class_Hex):
            result_intbase = self.int_base - other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base - other
        elif isinstance(other, int):
            result_intbase = self.int_base - other
        else:
            return NotImplemented
        
        return Class_Hex(result_intbase)
    
    def __mul__(self, other):
        """Multiplicación delegada a IntBase"""
        if isinstance(other, Class_Hex):
            result_intbase = self.int_base * other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base * other
        elif isinstance(other, int):
            result_intbase = self.int_base * other
        else:
            return NotImplemented
        
        return Class_Hex(result_intbase)
    
    def __truediv__(self, other):
        """División delegada a IntBase"""
        if isinstance(other, Class_Hex):
            result_intbase = self.int_base / other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base / other
        elif isinstance(other, int):
            result_intbase = self.int_base / other
        else:
            return NotImplemented
        
        return Class_Hex(result_intbase)
    
    def __mod__(self, other):
        """Módulo delegado a IntBase"""
        if isinstance(other, Class_Hex):
            result_intbase = self.int_base % other.int_base
        elif isinstance(other, IntBase):
            result_intbase = self.int_base % other
        elif isinstance(other, int):
            result_intbase = self.int_base % other
        else:
            return NotImplemented
        
        return Class_Hex(result_intbase)
    
    # Operadores reversos
    def __radd__(self, other):
        return self.__add__(other)
    
    def __rmul__(self, other):
        return self.__mul__(other)
    
    # ========== MÉTODOS ESTÁTICOS ==========
    
    @staticmethod
    def Helper(input_str):
        """Ayuda contextual para Hex"""
        if re.match(r"^\s*Hex\b", input_str, re.IGNORECASE):
            return ('Ej: 16#FF (se convierte automáticamente), Hex(255)\n'
                   'Funciones: toDecimal(), toBinary(), toOctal(), toBase(n)')
        return None

    @staticmethod
    def PopupFunctionList():
        """Lista de métodos sugeridos para autocompletado de Hex"""
        return [
            ("toDecimal", "Convierte a decimal"),
            ("toBinary", "Convierte a binario"),
            ("toOctal", "Convierte a octal"),
            ("toBase", "Convierte a base específica"),
        ]

    @staticmethod
    def get_tokenization_patterns():
        """
        Define patrones adicionales de tokenización específicos para Hex
        
        Returns:
            List[Dict]: Lista de reglas de tokenización (opcional)
        """
        return [
            {
                'pattern': r'\b0x([0-9A-Fa-f]+)\b',
                'replacement': lambda match: f'Hex("{match.group(1)}")',
                'priority': 6,  # Prioridad media, después de IntBase pero específica
                'description': 'Hexadecimal con prefijo 0x: 0xFF, 0xA0'
            }
        ]


# ========== FUNCIÓN DE REGISTRO ==========

def register_classes_in_module():
    """
    Devuelve una lista de clases definidas en este módulo para ser registradas.
    NOTA: Ya no se registra para corchetes, el tokenizador maneja 16#valor automáticamente
    """
    return [
        ("Hex", Class_Hex, "SympyClassBase", {
            "add_lowercase": True,
            "supports_brackets": False,  # Ya no usa corchetes
            "description": "Números hexadecimales (usar 16#valor para tokenización automática)"
        }),
    ]