"""
Clase base universal para números en cualquier base - TIPO REGISTRADO
"""
from class_base import ClassBase
import sympy
import re

class IntBase(ClassBase):
    """
    Representación universal de números en cualquier base
    con manejo interno de símbolos y conversión perezosa a SymPy
    """
    
    def __init__(self, value_str, base=10):
        self.value_str = str(value_str)  
        self.base = int(base)
        self.has_symbols = bool(re.search(r'[a-zA-Z_]', self.value_str))
        
        if self.has_symbols:
            # Modo algebraico: mantener símbolos INTERNAMENTE, no convertir a SymPy aún
            self._symbols = self._extract_symbols()
            self._numeric_value = None  # No hay valor numérico directo
            original_str = f"{self.base}#{self.value_str}"
            super().__init__(self.value_str, original_str)  # ClassBase, NO SympyClassBase
        else:
            # Modo numérico: convertir a entero
            try:
                self._numeric_value = int(self.value_str, self.base)
                self._symbols = []
                original_str = f"{self.base}#{self.value_str}"
                super().__init__(self._numeric_value, original_str)  # ClassBase
            except ValueError:
                raise ValueError(f"Valor inválido '{self.value_str}' para base {self.base}")
    
    def _extract_symbols(self):
        """Extrae símbolos de la cadena, manteniendo caracteres únicos"""
        return list(set(re.findall(r'[a-zA-Z_][a-zA-Z0-9_]*', self.value_str)))
    
    def substitute(self, **kwargs):
        """Sustitución NATIVA - mantiene como IntBase"""
        if not self.has_symbols:
            return self  # Sin símbolos, retornar el mismo objeto
        
        new_value_str = self.value_str
        for symbol, value in kwargs.items():
            if symbol in self._symbols:
                new_value_str = new_value_str.replace(symbol, str(value))
        
        # Retornar nuevo IntBase con valores sustituidos
        return IntBase(new_value_str, self.base)
    
    def to_sympy(self):
        """CONVERSIÓN EXPLÍCITA a SymPy cuando se necesite álgebra compleja"""
        if self.has_symbols:
            # Construir expresión SymPy paso a paso
            result = 0
            digits = list(self.value_str)
            
            for i, char in enumerate(reversed(digits)):
                power = self.base ** i
                
                if char.isdigit():
                    coefficient = int(char)
                elif char.upper() in 'ABCDEF' and self.base == 16:
                    coefficient = ord(char.upper()) - ord('A') + 10
                elif re.match(r'[a-zA-Z_]', char):
                    # Es un símbolo
                    coefficient = sympy.Symbol(char)
                else:
                    raise ValueError(f"Carácter inválido '{char}' en valor '{self.value_str}'")
                
                result += coefficient * power
            
            return result
        else:
            # Sin símbolos, retornar entero SymPy simple
            return sympy.sympify(self._numeric_value)
    
    def to_base(self, new_base):
        """Convierte a nueva base"""
        if self.has_symbols:
            # En modo algebraico, mantener expresión
            return IntBase(f"({self.value_str})_base{new_base}", new_base)
        else:
            # En modo numérico, convertir directamente
            new_value_str = self._convert_to_base_string(self._numeric_value, new_base)
            return IntBase(new_value_str, new_base)
    
    def to_decimal(self):
        """Obtiene valor decimal"""
        if self.has_symbols:
            return f"decimal({self.value_str}, base={self.base})"
        else:
            return self._numeric_value
    
    def to_hex(self):
        """Convierte a hexadecimal"""
        return self.to_base(16)
    
    def to_binary(self):
        """Convierte a binario"""
        return self.to_base(2)
    
    def to_octal(self):
        """Convierte a octal"""
        return self.to_base(8)
    
    @property
    def value(self):
        """Propiedad de acceso al valor - mantiene nativo"""
        if self.has_symbols:
            return self.value_str  # Mantener como string simbólico
        else:
            return self._numeric_value  # Valor numérico directo
    
    # ========== OPERADORES ARITMÉTICOS NATIVOS (mantienen IntBase) ==========
    
    def __add__(self, other):
        """Suma nativa - mantiene como IntBase cuando es posible"""
        if self.has_symbols:
            # Con símbolos, crear expresión simbólica pero mantener IntBase
            if isinstance(other, IntBase):
                if other.has_symbols:
                    new_expr = f"({self.value_str}) + ({other.value_str})"
                else:
                    new_expr = f"({self.value_str}) + {other._numeric_value}"
            elif isinstance(other, (int, float)):
                new_expr = f"({self.value_str}) + {other}"
            else:
                # Para otros tipos, convertir a SymPy
                return self.to_sympy() + other
            
            return IntBase(new_expr, self.base)
        
        # Sin símbolos: aritmética numérica directa
        if isinstance(other, IntBase):
            if other.has_symbols:
                # El otro tiene símbolos, crear expresión
                new_expr = f"{self._numeric_value} + ({other.value_str})"
                return IntBase(new_expr, self.base)
            else:
                # Ambos numéricos
                result_value = self._numeric_value + other._numeric_value
        elif isinstance(other, (int, float)):
            result_value = self._numeric_value + other
        else:
            # Para tipos complejos, convertir a SymPy
            return self.to_sympy() + other
        
        # Convertir resultado de vuelta a cadena en la base original
        result_str = self._convert_to_base_string(int(result_value), self.base)
        return IntBase(result_str, self.base)
    
    def __sub__(self, other):
        """Resta nativa"""
        if self.has_symbols:
            if isinstance(other, IntBase):
                if other.has_symbols:
                    new_expr = f"({self.value_str}) - ({other.value_str})"
                else:
                    new_expr = f"({self.value_str}) - {other._numeric_value}"
            elif isinstance(other, (int, float)):
                new_expr = f"({self.value_str}) - {other}"
            else:
                return self.to_sympy() - other
            
            return IntBase(new_expr, self.base)
        
        # Sin símbolos
        if isinstance(other, IntBase):
            if other.has_symbols:
                new_expr = f"{self._numeric_value} - ({other.value_str})"
                return IntBase(new_expr, self.base)
            else:
                result_value = self._numeric_value - other._numeric_value
        elif isinstance(other, (int, float)):
            result_value = self._numeric_value - other
        else:
            return self.to_sympy() - other
        
        result_str = self._convert_to_base_string(int(result_value), self.base)
        return IntBase(result_str, self.base)
    
    def __mul__(self, other):
        """Multiplicación nativa"""
        if self.has_symbols:
            if isinstance(other, IntBase):
                if other.has_symbols:
                    new_expr = f"({self.value_str}) * ({other.value_str})"
                else:
                    new_expr = f"({self.value_str}) * {other._numeric_value}"
            elif isinstance(other, (int, float)):
                new_expr = f"({self.value_str}) * {other}"
            else:
                return self.to_sympy() * other
            
            return IntBase(new_expr, self.base)
        
        # Sin símbolos
        if isinstance(other, IntBase):
            if other.has_symbols:
                new_expr = f"{self._numeric_value} * ({other.value_str})"
                return IntBase(new_expr, self.base)
            else:
                result_value = self._numeric_value * other._numeric_value
        elif isinstance(other, (int, float)):
            result_value = self._numeric_value * other
        else:
            return self.to_sympy() * other
        
        result_str = self._convert_to_base_string(int(result_value), self.base)
        return IntBase(result_str, self.base)
    
    @staticmethod
    def _convert_to_base_string(value, base):
        """Convierte un entero a string en la base especificada"""
        if base == 2:
            return bin(value)[2:]
        elif base == 8:
            return oct(value)[2:]
        elif base == 16:
            return hex(value)[2:].upper()
        elif base == 10:
            return str(value)
        else:
            # Algoritmo general para cualquier base
            if value == 0:
                return "0"
            
            digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
            result = ""
            
            while value > 0:
                result = digits[value % base] + result
                value //= base
            
            return result
    
    def __str__(self):
        """Representación como cadena"""
        return f"{self.base}#{self.value_str}"
    
    def __repr__(self):
        """Representación para debug"""
        return f"IntBase('{self.value_str}', {self.base})"
    
    @staticmethod
    def Helper(input_str):
        """Ayuda contextual para IntBase"""
        if re.search(r'\d+#[0-9A-Fa-fx]+', input_str):
            return '''IntBase - Números en cualquier base con álgebra simbólica
            
Sintaxis: base#valor
Ejemplos:
  16#FF        → IntBase('FF', 16) = 255
  2#1010       → IntBase('1010', 2) = 10
  8#777        → IntBase('777', 8) = 511
  16#x0        → IntBase('x0', 16) = simbólico
  
Operaciones aritméticas mantienen la base original:
  16#FF + 16#10  → IntBase('10F', 16)
  2#1010 * 3     → IntBase('11110', 2)
  
Métodos disponibles:
  .to_base(nueva_base)  - Convierte a otra base
  .substitute(x=valor)  - Sustituye símbolos
  .to_sympy()          - Conversión explícita a SymPy'''
        return None
    
    @staticmethod
    def PopupFunctionList():
        """Lista de métodos para autocompletado"""
        return [
            ("to_base", "Convierte a otra base"),
            ("substitute", "Sustituye símbolos por valores"),
            ("to_sympy", "Conversión explícita a SymPy para álgebra"),
            ("to_decimal", "Obtiene valor decimal"),
            ("to_hex", "Convierte a hexadecimal"),
            ("to_binary", "Convierte a binario"),
            ("to_octal", "Convierte a octal"),
        ]
    
    @staticmethod
    def get_tokenization_patterns():
        """
        Define cómo esta clase debe ser reconocida y tokenizada
        
        Returns:
            List[Dict]: Lista de reglas de tokenización
        """
        return [
            {
                'pattern': r'(\d+)#([0-9A-Fa-fx]+)',
                'replacement': lambda match: f'IntBase("{match.group(2)}", {match.group(1)})',
                'priority': 5,  # ALTA prioridad (patrón muy específico)
                'description': 'Números con base: 16#FF, 2#1010, etc.'
            }
        ]

def register_classes_in_module():
    """Registro de IntBase en el sistema de tipos"""
    return [
        ("IntBase", IntBase, "ClassBase", {
            "add_lowercase": True,
            "supports_brackets": False,  # Se maneja por tokenización
            "is_fundamental": True,      # Clase fundamental del sistema
            "description": "Números universales en cualquier base con álgebra simbólica"
        }),
    ] 